@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: typedecl.ml,v 1.48.2.8.2.1 2009/06/19 07:11:12 maranget Exp $ *)
+(* $Id: typedecl.ml 9592 2010-01-25 14:57:26Z maranget $ *)
 
 (**** Typing of type definitions ****)
 
@@ -184,11 +184,7 @@ let transl_declaration env (name, sdecl) id =
           None -> None
         | Some sty ->
             let no_row = not (is_fixed_type sdecl) in
-            let ty =
-              transl_simple_type env no_row sty in
-            if Ctype.cyclic_abbrev env id ty then
-              raise(Error(sdecl.ptype_loc, Recursive_abbrev name));
-            Some ty
+            Some (transl_simple_type env no_row sty)
         end;
       type_variance = List.map (fun _ -> true, true, true) params;
     } in
@@ -200,12 +196,19 @@ let transl_declaration env (name, sdecl) id =
         raise(Error(loc, Unconsistent_constraint tr)))
     cstrs;
   Ctype.end_def ();
+  (* Add abstract row *)
   if is_fixed_type sdecl then begin
     let (p, _) =
       try Env.lookup_type (Longident.Lident(Ident.name id ^ "#row")) env
       with Not_found -> assert false in
     set_fixed_row env sdecl.ptype_loc p decl
   end;
+  (* Check for cyclic abbreviations *)
+  begin match decl.type_manifest with None -> ()
+  | Some ty ->
+      if Ctype.cyclic_abbrev env id ty then
+        raise(Error(sdecl.ptype_loc, Recursive_abbrev name));
+  end;
   (id, decl)
 
 (* Generalize a type declaration *)
